//
// NETDEMO
// Copyright (c) 2003 Rama C. Hoetzlein
//

#ifndef NET_ENG
	#define NET_ENG

	#include "netdemo-buffer.h"
	#include "netdemo-sock.h"

	#define NET_BUFSIZE			8000	// Buffer size for input
		// (Normally, you would >not< create a buffer for UDP. UDP is designed to be unreliable but fast)

	#define NET_NO_MODE			0		// Current mode of operation. Client or server.
	#define NET_CLIENT			1
	#define NET_SERVER			2
	
	#define NET_OFF				0		// Client & Server States
	#define NET_START			1
	#define NET_RESET			2
	#define NET_DISCONNECTED	3
	#define NET_SEARCHING		4		
	#define NET_CONNECTING		5
	#define NET_CONNECTED		6
	#define NET_PROCESSING		7

	#define NET_UDP				0		// Request UDP protocol (TCP not yet supported)

	#define NET_BROADCAST_OFF	0		// Request broadcasting off for a socket
	#define NET_BROADCAST_ON	1		// Request broadcasting on for a socket

	#define NET_SEARCH_PORT		8000	// Server port number 
		// (This is the base port number of server. Clients will each recieve a
		//  port number incrementally from this base port number.)

	class NetEng {
	public:
		NetEng();
		
		// Application-level functions
		void	Start (char m);						// Start net engine in client or server mode
		void	Go (void);							// Process current state
		int		SendByteToServer (char c);			// Send byte to the server
		int		SendByteToClients (char c);			// Send byte to all clients
		int		RecieveByte (int* who);				// Recieve byte (from server or client)
		int		GetNumClients (void) {return num_net_sockets-1;}	// Get number of clients (server only)		
		
		// State functions
		void	ClientStart (void);
		void	ClientReset (void);
		void	ClientDisconnected (void);
		void	ClientSearching (void);
		void	ClientConnecting (void);
		void	ClientConnected (void);		
		void	ServerStart (void);
		void	ServerReset (void);
		void	ServerDisconnected (void);
		void	ServerProcessing (void);

		// Socket manipulation
		int		AddSocket (char protocol);
		int		FindSocketWithSrcPort (int chk_port);
		
		// Single Socket functions			
		inline void SetSocketDest (int sn, NetAddress &d)	{net_sockets[sn].SetDest (d);}
		inline void SetSocketDestPort (int sn, int p)		{net_sockets[sn].SetDestPort (p);}
		inline void	SetSocketDestBroadcast (int sn, int p)	{net_sockets[sn].SetDestBroadcast (p);}
		inline NetAddress &GetSocketDest (int sn)			{return net_sockets[sn].GetDest ();}
		inline void GetSocketDestName (int sn, char* buf, unsigned long* buflen)	{net_sockets[sn].GetDestName (buf, buflen);}
		
		inline void SetSocketSrc (int sn, NetAddress &d)	{net_sockets[sn].SetSrc (d);}
		inline void SetSocketSrcPort (int sn, int p)		{net_sockets[sn].SetSrcPort (p);}
		inline void SetSocketSrcAny (int sn, int p)			{net_sockets[sn].SetSrcAny (p);}
		inline void SetSocketSrcToDest (int sn1, int sn2)	{net_sockets[sn2].SetDest (net_sockets[sn1].GetSrc());}
		inline NetAddress &GetSocketSrc (int sn)			{return net_sockets[sn].GetSrc ();}		
		inline void GetSocketSrcName (int sn, char* buf, unsigned long* buflen)	{net_sockets[sn].GetSrcName (buf, buflen);}
		
		inline void	SetSocketMode (int sn, char m)			{net_sockets[sn].SetMode (m);}
		inline void SetSocketTimeout (int sn, int sec, int msec) {net_sockets[sn].SetTimeout (sec, msec);}
		inline void SetSocketSendTime (int sn, int t)		{net_sockets[sn].SetSendTime (t);}
		inline int GetSocketSendTime (int sn)				{return net_sockets[sn].GetSendTime ();}
		
		inline int SocketSendDataTo (int sn, char *dat, int len)	{return net_sockets[sn].SendDataTo (dat, len);}
		inline int SocketRecvDataFrom (int sn, char *dat, int len) {return net_sockets[sn].RecvDataFrom (dat, len);}
		inline int SocketSendData (int sn, char *dat, int len)	{return net_sockets[sn].SendData (dat, len);}
		inline int SocketRecvData (int sn, char *dat, int len) {return net_sockets[sn].RecvData (dat, len);}
		inline void SocketBindToSrc (int sn)				{net_sockets[sn].BindToSrc ();}
		inline void SocketBindToDest (int sn)				{net_sockets[sn].BindToDest ();}
		inline void SocketConnectToSrc (int sn)				{net_sockets[sn].ConnectToSrc ();}
		inline void SocketConnectToDest (int sn)			{net_sockets[sn].ConnectToDest ();}

		inline void SetSocketConnected (int sn, bool c)		{net_sockets[sn].SetConnected (c);}
		inline bool GetSocketConnected (int sn)				{return net_sockets[sn].GetConnected ();}

		char*	GetDestName (int sn);
		char*	GetSrcName (int sn);

		// Multi-socket Functions for Send/Recieve
		int RecieveFrom (void);
		int Recieve (void);		
		int SendTo (int sn, char* buf, int len);
		int Send (int sn, char* buf, int len);

	private:
		char			mode;					// Client or server?
		char			state;					// Current state of client/server
		
		int				max_net_sockets;		// Maximum sockets currently permitted
		int				num_net_sockets;		// Current number of sockets (always 1 for client, many for server)
		NetSock*		net_sockets;			// Pointer to list of sockets
		struct fd_set	socket_set;				// Socket set used to poll socket input 
		unsigned int	serv_port;				// Servers port number (maintained for server)
		unsigned int	client_sn;				// Clients socket number on server (maintained for client) 

		char			addrname[100];			// Buffer for human-readable network addresses
		unsigned long	addrlen;

												//** Low-level buffer
		Buffer			data;	// Low level buffer is used to make sure all data is received. 
								// This not a recommended use of UDP as it makes the
								// connection 'reliable'. Low level buffer includes
								// connections messages and application level data.

												//** High-level buffer
								// High level buffer is application-level data only.
		char			curr_byte;				// Buffer for current send/recv data
		bool			byte_available;			// Indicates if send/recv buffer is full
		int				from_who;				// Indicates who filled the send/recv buffer		
	};

#endif

